<?php

namespace Drupal\pagedesigner;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Language\LanguageManager;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Service\ElementHandler;

/**
 * Base class for handling operations on elements and content entities.
 */
class PagedesignerService {
  /**
   * The output of the last operation.
   *
   * @var mixed
   */
  protected $_output = NULL;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager = NULL;

  /**
   * The element handler.
   *
   * @var \Drupal\pagedesigner\Service\ElementHandler
   */
  protected $elementHandler = NULL;

  /**
   * Create a new instance.
   *
   * @param \Drupal\Core\Language\LanguageManager $language_manager
   *   The language manager.
   * @param \Drupal\pagedesigner\Entity\ElementHandler $element_handler
   *   The element handler.
   */
  public function __construct(LanguageManager $language_manager, ElementHandler $element_handler) {
    $this->languageManager = $language_manager;
    $this->elementHandler = $element_handler;
  }

  /**
   * Returns the output of the last operation.
   *
   * @return mixed
   *   The output of the last operation.
   */
  public function getOutput() {
    return $this->_output;
  }

  /**
   * Get all pagedesigner fields of the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to get the fields from.
   *
   * @return array
   *   The list of fields, keyed by field name.
   */
  public function getPagedesignerFields(ContentEntityBase $entity) {
    $pagedesignerFields = [];
    $fields = $entity->getFields(TRUE);
    foreach ($fields as $field) {
      if ($field->getFieldDefinition()->getType() == 'pagedesigner_item') {
        $pagedesignerFields[$field->getName()] = $field;
      }
    }
    return $pagedesignerFields;
  }

  /**
   * Adds the container value to the given entity.
   *
   * If no container exists, a container will be created.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to add the containers to.
   */
  public function addContainer(ContentEntityBase &$entity) {
    $fields = $this->getPagedesignerFields($entity);
    foreach ($fields as $field) {
      if ($entity->get($field->getName())->isEmpty()) {
        $container = $this->getContainer($entity, $field->getName());
        $entity->get($field->getName())->entity = $container;
      }
      elseif (!$entity->isDefaultTranslation()) {
        $container = $entity->get($field->getName())->entity;
        $language = $entity->language()->getId();
        if (!$container->hasTranslation($language)) {
          $container->addTranslation($language);
          $container->save();
        }

      }
    }
  }

  /**
   * Replaces the container linked in the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to replace the container in.
   */
  public function overrideContainers(ContentEntityBase &$entity) {
    $fields = $this->getPagedesignerFields($entity);
    foreach ($fields as $field) {
      $container = $pagedesignerService->createContainer($entity->language()->getId());
      $entity->getUntranslated()->get($field->getName())->entity = $container;
    }
    foreach ($entity->getTranslationLanguages(FALSE) as $language) {
      foreach ($fields as $field) {
        $entity->getTranslation($language->getId())->get($field->getName())->entity = $entity->getUntranslated()->get($field->getName())->entity;
      }
    }
  }

  /**
   * Saves the containers in all fields.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   */
  public function saveContainer(ContentEntityBase &$entity) {
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $fields = $entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($fields as $field) {
      if ($field->getType() == 'pagedesigner_item') {
        if (!$entity->get($field->getName())->isEmpty()) {
          $container = $entity->get($field->getName())->entity;
          $container->entity = $entity;
          $container->save();
        }
      }
    }

  }

  /**
   * Loads or creates the container for the given entity in the language of the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity being rendered.
   *
   * @return \Drupal\pagedesigner\Entity\Element
   *   The container.
   */
  public function getContainer(ContentEntityBase $entity, $fieldname = 'field_pagedesigner_content') {
    $container = $entity->get($fieldname)->entity;
    $language = $entity->language()->getId();
    if ($container == NULL) {
      $container = $this->createContainer($language);
    }
    else {
      $container = $this->getLanguage($container, $language);
    }
    return $container;
  }

  /**
   * Create a new container.
   *
   * @return \Drupal\pagedesigner\Entity\Element
   *   The container.
   */
  public function createContainer($language) {
    $container = Element::create(['type' => 'container', 'name' => 'container']);
    $container->langcode->value = $language;
    $container->save();
    return $container;
  }

  /**
   * Gets or creates the translation of the given container.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container.
   * @param string $language
   *   The language.
   *
   * @return \Drupal\pagedesigner\Entity\Element
   *   The container in the given language.
   */
  public function getLanguage(Element $container, $language) {
    if (!$container->hasTranslation($language)) {
      $container->addTranslation($language);
      $container->save();
    }
    return $container->getTranslation($language);
  }

  /**
   * Returns true if the edit mode of the pagedesigner is active.
   *
   * @return bool
   */
  public function isEditMode() {
    return \Drupal::routeMatch()->getRouteName() == 'pagedesigner.node.edit_mode';
  }

  /**
   * Get the handler.
   *
   * @return Drupal\pagedesigner\Service\ElementHandler
   */
  protected function getHandler() {
    return $this->elementHandler;
  }

}
